# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2024, by Samuel Williams.

source "https://rubygems.org"

gem "async"
gem "async-http", path: "../../../async-http"
gem "protocol-http", path: "../../"

gem "debug"
