# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2018-2024, by Samuel Williams.

require_relative "http/version"

require_relative "http/headers"
require_relative "http/request"
require_relative "http/response"
require_relative "http/middleware"

# @namespace
module Protocol
	# @namespace
	module HTTP
	end
end
