# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2024, by Samuel Williams.

require_relative "body/readable"
require_relative "body/writable"
require_relative "body/wrapper"

module Protocol
	module HTTP
		# @namespace
		module Body
		end
	end
end
